@@ -151,16 +151,15 @@ pub(crate) fn add_to_dunder_all<'a>(
     stylist: &Stylist,
 ) -> Vec<Edit> {
     let (insertion_point, export_prefix_length) = match expr {
-        Expr::List(ExprList { elts, range, .. }) => (
-            elts.last()
-                .map_or(range.end() - "]".text_len(), Ranged::end),
+        Expr::List(ExprList { elts, .. }) => (
+            elts.last().map_or(expr.end() - "]".text_len(), Ranged::end),
             elts.len(),
         ),
         Expr::Tuple(tup) if tup.parenthesized => (
             tup.elts
                 .last()
                 .map_or(tup.end() - ")".text_len(), Ranged::end),
-            tup.elts.len(),
+            tup.len(),
         ),
         Expr::Tuple(tup) if !tup.parenthesized => (
             tup.elts
@@ -168,7 +167,7 @@ pub(crate) fn add_to_dunder_all<'a>(
                 .expect("unparenthesized empty tuple is not possible")
                 .range()
                 .end(),
-            tup.elts.len(),
+            tup.len(),
         ),
         _ => {
             // we don't know how to insert into this expression
@@ -318,26 +317,28 @@ pub(crate) fn adjust_indentation(
         line_indentation.contains('\t') && line_indentation.contains(' ')
     });
 
-    if contains_multiline_string || mixed_indentation {
-        let module_text = format!("def f():{}{contents}", stylist.line_ending().as_str());
-
-        let mut tree = match_statement(&module_text)?;
-
-        let embedding = match_function_def(&mut tree)?;
-
-        let indented_block = match_indented_block(&mut embedding.body)?;
-        indented_block.indent = Some(indentation);
-
-        let module_text = indented_block.codegen_stylist(stylist);
-        let module_text = module_text
-            .strip_prefix(stylist.line_ending().as_str())
-            .unwrap()
-            .to_string();
-        Ok(module_text)
-    } else {
-        // Otherwise, we can do a simple adjustment ourselves.
-        Ok(dedent_to(contents, indentation))
+    // For simple cases, try to do a manual dedent.
+    if !contains_multiline_string && !mixed_indentation {
+        if let Some(dedent) = dedent_to(contents, indentation) {
+            return Ok(dedent);
+        }
     }
+
+    let module_text = format!("def f():{}{contents}", stylist.line_ending().as_str());
+
+    let mut tree = match_statement(&module_text)?;
+
+    let embedding = match_function_def(&mut tree)?;
+
+    let indented_block = match_indented_block(&mut embedding.body)?;
+    indented_block.indent = Some(indentation);
+
+    let module_text = indented_block.codegen_stylist(stylist);
+    let module_text = module_text
+        .strip_prefix(stylist.line_ending().as_str())
+        .unwrap()
+        .to_string();
+    Ok(module_text)
 }
 
 /// Determine if a vector contains only one, specific element.
