@@ -26,17 +26,17 @@ use crate::checkers::ast::Checker;
 /// these comparison operators -- `__eq__` and `__ne__`.
 ///
 /// ## Example
+///
 /// ```python
 /// class Foo:
-///     def __eq__(self, obj: typing.Any) -> bool:
-///         ...
+///     def __eq__(self, obj: typing.Any) -> bool: ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
 /// class Foo:
-///     def __eq__(self, obj: object) -> bool:
-///         ...
+///     def __eq__(self, obj: object) -> bool: ...
 /// ```
 /// ## References
 /// - [Python documentation: The `Any` type](https://docs.python.org/3/library/typing.html#the-any-type)
@@ -72,11 +72,13 @@ pub(crate) fn any_eq_ne_annotation(checker: &mut Checker, name: &str, parameters
         return;
     };
 
-    if !checker.semantic().current_scope().kind.is_class() {
+    let semantic = checker.semantic();
+
+    if !semantic.current_scope().kind.is_class() {
         return;
     }
 
-    if checker.semantic().match_typing_expr(annotation, "Any") {
+    if semantic.match_typing_expr(annotation, "Any") {
         let mut diagnostic = Diagnostic::new(
             AnyEqNeAnnotation {
                 method_name: name.to_string(),
@@ -84,12 +86,15 @@ pub(crate) fn any_eq_ne_annotation(checker: &mut Checker, name: &str, parameters
             annotation.range(),
         );
         // Ex) `def __eq__(self, obj: Any): ...`
-        if checker.semantic().is_builtin("object") {
-            diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
-                "object".to_string(),
-                annotation.range(),
-            )));
-        }
+        diagnostic.try_set_fix(|| {
+            let (import_edit, binding) = checker.importer().get_or_import_builtin_symbol(
+                "object",
+                annotation.start(),
+                semantic,
+            )?;
+            let binding_edit = Edit::range_replacement(binding, annotation.range());
+            Ok(Fix::safe_edits(binding_edit, import_edit))
+        });
         checker.diagnostics.push(diagnostic);
     }
 }
