@@ -49,7 +49,6 @@ mod tests {
     #[test_case(Rule::UnsortedDunderAll, Path::new("RUF022.py"))]
     #[test_case(Rule::UnsortedDunderSlots, Path::new("RUF023.py"))]
     #[test_case(Rule::MutableFromkeysValue, Path::new("RUF024.py"))]
-    #[test_case(Rule::UnnecessaryDictComprehensionForIterable, Path::new("RUF025.py"))]
     #[test_case(Rule::DefaultFactoryKwarg, Path::new("RUF026.py"))]
     #[test_case(Rule::MissingFStringSyntax, Path::new("RUF027_0.py"))]
     #[test_case(Rule::MissingFStringSyntax, Path::new("RUF027_1.py"))]
@@ -84,6 +83,22 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn no_remove_parentheses_starred_expr_py310() -> Result<()> {
+        let diagnostics = test_path(
+            Path::new("ruff/RUF031.py"),
+            &LinterSettings {
+                ruff: super::settings::Settings {
+                    parenthesize_tuple_in_subscript: false,
+                },
+                target_version: PythonVersion::Py310,
+                ..LinterSettings::for_rule(Rule::IncorrectlyParenthesizedTupleInSubscript)
+            },
+        )?;
+        assert_messages!(diagnostics);
+        Ok(())
+    }
+
     #[test_case(Path::new("RUF013_0.py"))]
     #[test_case(Path::new("RUF013_1.py"))]
     fn implicit_optional_py39(path: &Path) -> Result<()> {
