@@ -1,27 +1,35 @@
-use memchr::memchr2_iter;
 use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Edit, Fix};
 use ruff_macros::{derive_message_formats, violation};
 use ruff_python_ast::{self as ast};
 use ruff_python_literal::format::FormatSpec;
 use ruff_python_parser::parse_expression;
+use ruff_python_semantic::analyze::logging;
 use ruff_python_semantic::SemanticModel;
 use ruff_source_file::Locator;
 use ruff_text_size::{Ranged, TextRange};
+
+use memchr::memchr2_iter;
 use rustc_hash::FxHashSet;
 
+use crate::checkers::ast::Checker;
+
 /// ## What it does
-/// Checks for strings that contain f-string syntax but are not f-strings.
+/// Searches for strings that look like they were meant to be f-strings, but are missing an `f` prefix.
 ///
 /// ## Why is this bad?
-/// An f-string missing an `f` at the beginning won't format anything, and instead
-/// treat the interpolation syntax as literal.
+/// Expressions inside curly braces are only evaluated if the string has an `f` prefix.
 ///
-/// Since there are many possible string literals which contain syntax similar to f-strings yet are not intended to be,
-/// this lint will disqualify any literal that satisfies any of the following conditions:
+/// ## Details
 ///
-/// 1. The string literal is a standalone expression. For example, a docstring.
-/// 2. The literal is part of a function call with argument names that match at least one variable (for example: `format("Message: {value}", value = "Hello World")`)
-/// 3. The literal (or a parent expression of the literal) has a direct method call on it (for example: `"{value}".format(...)`)
+/// There are many possible string literals which are not meant to be f-strings
+/// despite containing f-string-like syntax. As such, this lint ignores all strings
+/// where one of the following conditions applies:
+///
+/// 1. The string is a standalone expression. For example, the rule ignores all docstrings.
+/// 2. The string is part of a function call with argument names that match at least one variable
+///    (for example: `format("Message: {value}", value="Hello World")`)
+/// 3. The string (or a parent expression of the string) has a direct method call on it
+///    (for example: `"{value}".format(...)`)
 /// 4. The string has no `{...}` expression sections, or uses invalid f-string syntax.
 /// 5. The string references variables that are not in scope, or it doesn't capture variables at all.
 /// 6. Any format specifiers in the potential f-string are invalid.
@@ -30,15 +38,15 @@ use rustc_hash::FxHashSet;
 ///
 /// ```python
 /// name = "Sarah"
-/// dayofweek = "Tuesday"
-/// msg = "Hello {name}! It is {dayofweek} today!"
+/// day_of_week = "Tuesday"
+/// print("Hello {name}! It is {day_of_week} today!")
 /// ```
 ///
 /// Use instead:
 /// ```python
 /// name = "Sarah"
-/// dayofweek = "Tuesday"
-/// msg = f"Hello {name}! It is {dayofweek} today!"
+/// day_of_week = "Tuesday"
+/// print(f"Hello {name}! It is {day_of_week} today!")
 /// ```
 #[violation]
 pub struct MissingFStringSyntax;
@@ -55,12 +63,9 @@ impl AlwaysFixableViolation for MissingFStringSyntax {
 }
 
 /// RUF027
-pub(crate) fn missing_fstring_syntax(
-    diagnostics: &mut Vec<Diagnostic>,
-    literal: &ast::StringLiteral,
-    locator: &Locator,
-    semantic: &SemanticModel,
-) {
+pub(crate) fn missing_fstring_syntax(checker: &mut Checker, literal: &ast::StringLiteral) {
+    let semantic = checker.semantic();
+
     // we want to avoid statement expressions that are just a string literal.
     // there's no reason to have standalone f-strings and this lets us avoid docstrings too
     if let ast::Stmt::Expr(ast::StmtExpr { value, .. }) = semantic.current_statement() {
@@ -71,20 +76,27 @@ pub(crate) fn missing_fstring_syntax(
     }
 
     // We also want to avoid expressions that are intended to be translated.
-    if semantic
-        .current_expressions()
-        .any(|expr| is_gettext(expr, semantic))
-    {
+    if semantic.current_expressions().any(|expr| {
+        is_gettext(expr, semantic)
+            || is_logger_call(expr, semantic, &checker.settings.logger_objects)
+    }) {
         return;
     }
 
-    if should_be_fstring(literal, locator, semantic) {
+    if should_be_fstring(literal, checker.locator(), semantic) {
         let diagnostic = Diagnostic::new(MissingFStringSyntax, literal.range())
             .with_fix(fix_fstring_syntax(literal.range()));
-        diagnostics.push(diagnostic);
+        checker.diagnostics.push(diagnostic);
     }
 }
 
+fn is_logger_call(expr: &ast::Expr, semantic: &SemanticModel, logger_objects: &[String]) -> bool {
+    let ast::Expr::Call(ast::ExprCall { func, .. }) = expr else {
+        return false;
+    };
+    logging::is_logger_candidate(func, semantic, logger_objects)
+}
+
 /// Returns `true` if an expression appears to be a `gettext` call.
 ///
 /// We want to avoid statement expressions and assignments related to aliases
@@ -119,7 +131,7 @@ fn is_gettext(expr: &ast::Expr, semantic: &SemanticModel) -> bool {
         .is_some_and(|qualified_name| {
             matches!(
                 qualified_name.segments(),
-                ["gettext", "gettext" | "ngettext"]
+                ["gettext", "gettext" | "ngettext"] | ["builtins", "_"]
             )
         })
 }
