@@ -5,7 +5,7 @@
 import Editor, { BeforeMount, Monaco } from "@monaco-editor/react";
 import { MarkerSeverity, MarkerTag } from "monaco-editor";
 import { useCallback, useEffect, useRef } from "react";
-import { Diagnostic } from "../pkg";
+import { Diagnostic } from "../pkg/ruff_wasm";
 import { Theme } from "./theme";
 
 export default function SourceEditor({
@@ -39,7 +39,9 @@ export default function SourceEditor({
         startColumn: diagnostic.location.column,
         endLineNumber: diagnostic.end_location.row,
         endColumn: diagnostic.end_location.column,
-        message: diagnostic.code ? `${diagnostic.code}: ${diagnostic.message}` : diagnostic.message,
+        message: diagnostic.code
+          ? `${diagnostic.code}: ${diagnostic.message}`
+          : diagnostic.message,
         severity: MarkerSeverity.Error,
         tags:
           diagnostic.code === "F401" || diagnostic.code === "F841"
